﻿using UnityEngine;
using System.Collections;
using System.Text;
using System;

/* /////////////////////////////////////////////////////////////////////////////////////////////////
https://www.patreon.com/user?u=33893211

Licensed under CC BY-SA after EarlyAccess ended. (see https://creativecommons.org/licenses/by-sa/4.0/)

///////////////////////////////////////////////////////////////////////////////////////////////// */

namespace PluginIdea
{
    class SendData
    {
        public float[] samples;
    }

    public class SystemAudioAction : MVRScript
    {

        private JSONStorableFloat sampleMultNum;
        private JSONStorableFloat rollingMaxSample;
        private FloatTrigger SystemRealTimeAudioAmpTrigger;

        private float[] samples;

        private float triggerTimer;
        private float triggerDur = 0.05f;

        public override void Init()
        {
            base.Init();

            //trigger
            SimpleTriggerHandler.LoadAssets();

            SystemRealTimeAudioAmpTrigger = new FloatTrigger(this, "SystemRealTimeAudioAmpTrigger");
            Utils.SetupButton(this, "Set Audio Action", SystemRealTimeAudioAmpTrigger.OpenPanel, false);

            sampleMultNum = Utils.SetupSliderFloat(this, "SampleMultNum", 1f, 0.3f, 5f, false);
            rollingMaxSample = Utils.SetupSliderFloat(this, "rollingMaxSample", 1f, 1f, 5f, false);

            Utils.SetupInfoText(this, "SystemAudioAction 1.0 \n\n" +
                "Set Audio Action: set the application target of audio response.\n\n" +
                "SampleMultNum: The multiplier of audio effect, the larger the multiplier, the stronger the effect.\n\n" +
                "RollingMaxSample: the maximum number of audio samples skipped each time. The larger the setting, the later the audio response.", 500, true);
        }

        private void Update()
        {
            triggerTimer += Time.deltaTime;
            if (triggerTimer > triggerDur)
            {
                triggerTimer = 0;

                try
                {
                    if (UDPReceive.Instance.dataReveived != null)
                    {
                        string sendDataStr = Encoding.Default.GetString(UDPReceive.Instance.dataReveived);
                        SendData sendData = JsonUtility.FromJson<SendData>(sendDataStr);
                        float amp = GetAmp(sendData.samples);
                        SystemRealTimeAudioAmpTrigger.Trigger(amp);
                    }
                }
                catch (System.Exception e)
                {
                    SuperController.LogError(e.Message);
                    return;
                }
            }
        }

        private float GetAmp(float[] samples)
        {
            this.samples = samples;

            float ampMultiplier = 0f;
            for (int j = 0; j < this.samples.Length; j++)
            {
                this.samples[j] /= this.rollingMaxSample.val + 1E-06f;
                this.samples[j] *= this.sampleMultNum.val;
                ampMultiplier += this.samples[j];
            }
            ampMultiplier /= (float)this.samples.Length;
            return ampMultiplier;
        }

        private void OnDestroy()
        {
            UDPReceive.Instance.IsAlive = false;
        }
    }
}

